; ---------------------------------------------------
; module description : used by project MixedCan 
; module file name   : Can.asm
; created            : 9/25/97 8:11:47 PM
; ---------------------------------------------------

; ---CAN registers ---
CTL_ST       equ 0EF00h              ; control/status register
BIT_TIM      equ 0EF04h              ; bit timing register
UP_GLM_L     equ 0EF08h              ; upper global mask long
LO_GLM_L     equ 0EFA0h              ; lower global mask long
MSG_CTL_1    equ 0EF10h              ; message control register 1
MSG_CTL_2    equ 0EF20h              ; message control register 2
MSG_CTL_3    equ 0EF30h              ; message control register 3
MSG_CTL_4    equ 0EF40h              ; message control register 4
MSG_CTL_5    equ 0EF50h              ; message control register 5
MSG_CTL_6    equ 0EF60h              ; message control register 6   
MSG_CTL_7    equ 0EF70h              ; message control register 7
MSG_CTL_8    equ 0EF80h              ; message control register 8
MSG_CTL_9    equ 0EF90h              ; message control register 9
MSG_CTL_10   equ 0EFA0h              ; message control register 10
MSG_CTL_11   equ 0EFB0h              ; message control register 11
MSG_CTL_12   equ 0EFC0h              ; message control register 12
MSG_CTL_13   equ 0EFD0h              ; message control register 13
MSG_CTL_14   equ 0EFE0h              ; message control register 14
MSG_CTL_15   equ 0EFF0h              ; message control register 15

; ---------------------------------------------------
CanSetup:
   push  ASMREG         ; save nTransmit on stack
; --- configure control_status register ---
   mov   r0, #0041h     ; CTL_ST <- 0000 0000 0100 0001
   mov   CTL_ST, r0     ; setting bit 6 (CCE) allows CPU access to the
                        ; bit timing reister, setting bit 0 (INIT)
                        ; starts the initialization of CAN controller

; --- configure bit timing register ---
   mov   r0, #2F00h     ; BIT_TIM <- 0010 1111 0000 0000
   mov   BIT_TIM, r0    ; TSEG1=15, TSEG2=2, SJW=0, BRP=0
                        ; Baud rate = 500 kb/s

; --- set global mask registers ---
   mov   r0, #0FFFFh    ; upper mask word -> 1111 1111 1111 1111
   mov   UP_GLM_L, r0   ; lower mask word -> 1111 1000 1111 1111
   mov   r0, #0F8FFh    ; all identifier bits of the incoming messages
   mov   LO_GLM_L, r0   ; are checked according to this configuration
                                          
; --- initialize message object 1 ---
   mov   r0, #5595h         ; MSG_CTL_1 <- 0101 0101 1001 0101 
   mov   MSG_CTL_1, r0      ; message object 1 is valid.
                    
; --- set arbitration registers for message ---
   mov   r0, ZEROS          ; object 1.
   mov   MSG_CTL_1 + 2, r0  ; message object 1 accepts messages with 
   mov   MSG_CTL_1 + 4, r0  ; the identifier 0 (all 29 bits are 0).

; --- set message configuration register ---
   pop   ASMREG             ; recall nTransmit
   cmp   ASMREG, ZEROS
   jmpr  cc_EQ, _cs01
   mov   r0, #1Ch           ; message object 1 to transmit
   jmpr  cc_UC, _cs02
_cs01:
   mov   r0, #14h           ; message object 1 to receive
_cs02:
   mov   MSG_CTL_1 + 6, r0  ; DLC (data length)=1 bytes,
                            ; DIR=nTransmit (0 or 1), XTD=1 (extended)

; --- initialize message objects 2-15 ---
   mov   r0, #5555h     ; initialize message objects 2-15 
   mov   MSG_CTL_2, r0
   mov   MSG_CTL_3, r0  ; this application uses only the message 
   mov   MSG_CTL_4, r0  ; object 1. Message objects 2-15 are
   mov   MSG_CTL_5, r0  ; declared invalid by reseting MSGVAL segment
   mov   MSG_CTL_6, r0  ; of the corresponding message control
   mov   MSG_CTL_7, r0  ; register.
   mov   MSG_CTL_8, r0
   mov   MSG_CTL_9, r0
   mov   MSG_CTL_10, r0
   mov   MSG_CTL_11, r0
   mov   MSG_CTL_12, r0
   mov   MSG_CTL_13, r0
   mov   MSG_CTL_14, r0
   mov   MSG_CTL_15, r0

; --- complete the initialization process ---
                        ; initialization of the CAN controller
   mov   r0, ZEROS      ; is completed. Clearing bit 6 (CCE)  
   mov   CTL_ST, r0     ; inhibits further access to the bit 
                        ; timing register, clearing bit 0 (INIT) 
   ret                  ; completes the initialization

; ---------------------------------------------------
CanSendMsg:
                              ; prepare transmit message object (2)
   mov r1, #0EAFFh            ; MSG_CTL_1 <- 1110 1010 1111 1111 
   mov MSG_CTL_1, r1          ; TXRQ=1, NEWDAT=1, CPUUPD=1

   mov  r0, ASMREG
   movb MSG_CTL_1 + 7h, rl0   ; place data in the message object

   mov r1, #0F7FFh            ; transmit
   mov MSG_CTL_1, r1          ; MSG_CTL_1 <- 1111 0111 1111 1111
                              ; CPUUPD=0   
   ret

; ---------------------------------------------------
CanCheckMsg:
   mov r0, MSG_CTL_1
   and r0, #0300h             ; if there is a message (NEWDAT=1)
   cmp r0, #0200h
   jmpa cc_NZ, no_msg   
   mov ASMREG, #1             ; set receive_flag
   ret
no_msg:                       ; if no message    
   mov ASMREG, #0             ; clear receive_flag
   ret

; ---------------------------------------------------
CanGetMsg:
   movb rl0, MSG_CTL_1 + 7    ; get the ascii character
   movb rh0, #0
   push r0                    ; save msg data on stack
   mov  r0, #0FDFFh           ; and clear NEWDAT (NEWDAT=0)
   mov  MSG_CTL_1, r0
   movb rl0, #7               ; signal the reception of message
   pop  ASMREG                ; send it to the host   
   ret
; ---------------------------------------------------

