;------------------------- D4REC.ASM -------------------------------
;
; This program demonstrates the use of remote requests to trigger
; message transmission. D4TR.ASM should be run in the other node.
; A remote request is transmitted as soon as the input connected
; to port P2.0 goes low. An LED connected to port P2.4 signals
; the transmission of remote request. The program then waits for
; the incoming message. The arrival of the incoming message is
; signalled by an LED connected to port P2.7.
; In this example, only one message object is needed which is 
; configured as a receive object in 'initialize_can' routine. Both
; the generation of remote request and the reception of message is
; performed by this object.
;-------------------------------------------------------------------




; constant declarations

CTL_ST       equ 0EF00h              ; control/status register
BIT_TIM      equ 0EF04h              ; bit timing register
UP_GLM_L     equ 0EF08h              ; upper global mask long
LO_GLM_L     equ 0EFA0h              ; lower global mask long
MSG_CTL_1    equ 0EF10h              ; message control register 1
MSG_CTL_2    equ 0EF20h              ; message control register 2
MSG_CTL_3    equ 0EF30h              ; message control register 3
MSG_CTL_4    equ 0EF40h              ; message control register 4
MSG_CTL_5    equ 0EF50h              ; message control register 5
MSG_CTL_6    equ 0EF60h              ; message control register 6   
MSG_CTL_7    equ 0EF70h              ; message control register 7
MSG_CTL_8    equ 0EF80h              ; message control register 8
MSG_CTL_9    equ 0EF90h              ; message control register 9
MSG_CTL_10   equ 0EFA0h              ; message control register 10
MSG_CTL_11   equ 0EFB0h              ; message control register 11
MSG_CTL_12   equ 0EFC0h              ; message control register 12
MSG_CTL_13   equ 0EFD0h              ; message control register 13
MSG_CTL_14   equ 0EFE0h              ; message control register 14
MSG_CTL_15   equ 0EFF0h              ; message control register 15
INPUT_BUTTON equ P2.0



   org 04000h

   calla cc_UC, initialize_port
   calla cc_UC, initialize_can

not_pressed:
   jb INPUT_BUTTON, not_pressed       ; wait till P2.0 goes low

   calla cc_UC, send_remote           ; send remote request
   calla cc_UC, receive_data          ; wait for data from CAN bus
   calla cc_UC, process_data
   jmpa cc_UC, not_pressed            ; loop back
   



initialize_port:

   bclr DP2.0                         ; configure P2.0 as input
   bset DP2.4                         ; configure P2.4 as output
   bset DP2.7                         ; configure P2.7 as output
   bset P2.4
   bset P2.7
   ret


initialize_can:

                      ; configure control_status register
   mov r0, #0041h     ; CTL_ST <- 0000 0000 0100 0001
   mov CTL_ST, r0     ; setting bit 6 (CCE) allows CPU access to the
                      ; bit timing reister, setting bit 0 (INIT)
                      ; starts the initialization of CAN controller

                      ; configure bit timing register
   mov r0, #2F00h     ; BIT_TIM <- 0010 1111 0000 0000
   mov BIT_TIM, r0    ; TSEG1=15, TSEG2=2, SJW=0, BRP=0
                      ; Baud rate = 500 kb/s

                      ; set global mask registers                    
   mov r0, #0FFFFh    ; upper mask word -> 1111 1111 1111 1111
   mov UP_GLM_L, r0   ; lower mask word -> 1111 1000 1111 1111
   mov r0, #0F8FFh    ; all identifier bits of the incoming messages
   mov LO_GLM_L, r0   ; are checked in this configuration.
                                          
   mov r0, #5555h     ; initialize message objects 2-15 
   mov MSG_CTL_2, r0  ; this application uses only the message 
   mov MSG_CTL_3, r0  ; object 1. Message objects 2-15 are declared
   mov MSG_CTL_4, r0  ; invalid by resetting MSGVAL segment of the
   mov MSG_CTL_5, r0  ; corresponding message control register.
   mov MSG_CTL_6, r0
   mov MSG_CTL_7, r0
   mov MSG_CTL_8, r0
   mov MSG_CTL_9, r0
   mov MSG_CTL_10, r0
   mov MSG_CTL_11, r0
   mov MSG_CTL_12, r0
   mov MSG_CTL_13, r0
   mov MSG_CTL_14, r0
   mov MSG_CTL_15, r0

                          ; initialize message object 1
   mov r0, #5595h         ; MSG_CTL_1 <- 0101 0101 1001 0101 
   mov MSG_CTL_1, r0      ; message object 1 is valid.

                          ; set arbitration registers for message
   mov r0, #0000h         ; object 1.
   mov MSG_CTL_1 + 2, r0  ; message object 1 accepts messages with 
   mov MSG_CTL_1 + 4, r0  ; the identifier 0 (all 29 bits are 0).

                          ; set message configuration register 
   mov r0, #04h           ; for message object 1.
   mov MSG_CTL_1 + 6, r0  ; DLC (data length)=0 bytes,
                          ; DIR=0 (receive), XTD=1 (extended)

                          ; initialization of the CAN controller 
   mov r0, #0000h         ; is completed. Clearing bit 6 (CCE) 
   mov CTL_ST, r0         ; inhibits further access to the 
                          ; bit timing register, clearing bit 0
   ret                    ; (INIT) completes the initialization.



send_remote:              ; send remote request

   push r0

   mov r0, #0E7FFh        ; MSG_CTL_1 <- 1110 0111 1111 1111 
   mov MSG_CTL_1, r0      ; TXRQ=1, CPUUPD=0
   movb rl0, #4           ; signal the transmission of remote 
   calla cc_UC, blink     ; request by blinking LED in P2.4

   pop r0
   ret


receive_data:

   mov r0, MSG_CTL_1      ; check message control register of 
   and r0, #0300h         ; object 1. See if NEWDAT=1  
   cmp r0, #0200h
   jmpa cc_NZ, receive_data
   ret



process_data:

   mov r0, #0FDFFh        ; if there is a message, clear NEWDAT
   mov MSG_CTL_1, r0   
   movb rl0, #7           ; signal the reception of message
   calla cc_UC, blink     ; by blinking LED in P2.7
   ret




#include "r66util.inc"
