;------------------------- D4_2.ASM -------------------------------
;
; D4_1.ASM and D4_2.ASM are similar to D4REC.ASM and D4_TR.ASM in
; that they use remote requests to trigger message transmission.
; However, the program pair D4_1.ASM / D4_2.ASM enable duplex
; transmission. In other words, either node can send remote requests
; and receive messages.
;
; The CAN unit transmits a remote request as soon as the input 
; connected to port P2.0 goes low.  It then waits for the incoming 
; message. The arrival of the incoming message is signalled by an 
; LED connected to port P2.7.
;
; Each node needs two message objects; one receive object and one
; transmit object. The receive object is responsible for generating
; the remote request and receiving the corresponding message. The
; transmit object, on the other hand, is reponsible for receiving
; the remote request and transmitting the corresponding message.
;
; Both programs use the method of polling to check whether a remote
; requset or a message has arrived.
;-------------------------------------------------------------------



; constant declarations

RMT_STATUS   equ  2000h
CTL_ST       equ 0EF00h              ; control/status register
BIT_TIM      equ 0EF04h              ; bit timing register
UP_GLM_L     equ 0EF08h              ; upper global mask long
LO_GLM_L     equ 0EFA0h              ; lower global mask long
MSG_CTL_1    equ 0EF10h              ; message control register 1
MSG_CTL_2    equ 0EF20h              ; message control register 2
MSG_CTL_3    equ 0EF30h              ; message control register 3
MSG_CTL_4    equ 0EF40h              ; message control register 4
MSG_CTL_5    equ 0EF50h              ; message control register 5
MSG_CTL_6    equ 0EF60h              ; message control register 6   
MSG_CTL_7    equ 0EF70h              ; message control register 7
MSG_CTL_8    equ 0EF80h              ; message control register 8
MSG_CTL_9    equ 0EF90h              ; message control register 9
MSG_CTL_10   equ 0EFA0h              ; message control register 10
MSG_CTL_11   equ 0EFB0h              ; message control register 11
MSG_CTL_12   equ 0EFC0h              ; message control register 12
MSG_CTL_13   equ 0EFD0h              ; message control register 13
MSG_CTL_14   equ 0EFE0h              ; message control register 14
MSG_CTL_15   equ 0EFF0h              ; message control register 15
INPUT_BUTTON equ P2.0



   org 04000h

   calla cc_UC, initialize_port
   calla cc_UC, initialize_can

   calla cc_UC, clr_rmt_status
is_pressed:
   jb INPUT_BUTTON, no_input          ; see if the button is pressed
   calla cc_UC, send_rmt_rqst         ; if so, send remote request
no_input:
   calla cc_UC, receiv_rmt_rqst       ; see if a remote request is
   cmpb rl0, #1                       ; received. 
   calla cc_EQ, send_msg              ; if so, respond. 
   calla cc_UC, receive_msg           ; see if a message is received
   calla cc_UC, get_rmt_status
   cmpb rl0, #0
   jmpr cc_EQ, is_pressed
   jmpr cc_UC, no_input


initialize_port:

   bclr DP2.0                         ; configure P2.0 as input
   bset DP2.4                         ; configure P2.4 as output
   bset DP2.7                         ; configure P2.7 as output
   bset P2.4
   bset P2.7
   ret



initialize_can:

                      ; configure control_status register
   mov r0, #0041h     ; CTL_ST <- 0000 0000 0100 0001
   mov CTL_ST, r0     ; setting bit 6 (CCE) allows CPU access to the
                      ; bit timing reister, setting bit 0 (INIT)
                      ; starts the initialization of CAN controller

                      ; configure bit timing register
   mov r0, #2F00h     ; BIT_TIM <- 0010 1111 0000 0000
   mov BIT_TIM, r0    ; TSEG1=15, TSEG2=2, SJW=0, BRP=0
                      ; Baud rate = 500 kb/s

                      ; set global mask registers                    
   mov r0, #0FFFFh    ; upper mask word -> 1111 1111 1111 1111
   mov UP_GLM_L, r0   ; lower mask word -> 1111 1000 1111 1111
   mov r0, #0F8FFh    ; all identifier bits of the incoming messages
   mov LO_GLM_L, r0   ; are checked in this configuration.
                                          
   mov r0, #5555h     ; initialize message objects 3-15. 
   mov MSG_CTL_3, r0  ; this application uses only the message 
   mov MSG_CTL_4, r0  ; objects 1 and 2. Message objects 3-15 are
   mov MSG_CTL_5, r0  ; declared invalid by resetting MSGVAL field
   mov MSG_CTL_6, r0  ; of the corresponding message control 
   mov MSG_CTL_7, r0  ; register.
   mov MSG_CTL_8, r0
   mov MSG_CTL_9, r0
   mov MSG_CTL_10, r0
   mov MSG_CTL_11, r0
   mov MSG_CTL_12, r0
   mov MSG_CTL_13, r0
   mov MSG_CTL_14, r0
   mov MSG_CTL_15, r0
                          
   mov r0, #5595h         ; initialize message objects 1 and 2.
   mov MSG_CTL_1, r0      ; MSG_CTL_n <- 0101 0101 1001 0101 
   mov MSG_CTL_2, r0      ; message objects 1 and 2 are valid.
                            
   mov r0, #0000h         ; set arbitration registers for message
   mov MSG_CTL_1 + 2, r0  ; object 1.
   mov r0, #0100h         ; message object 1 accepts messages with 
   mov MSG_CTL_1 + 4, r0  ; the identifier 2.

                          ; set message configuration register 
   mov r0, #04h           ; for message object 1.
   mov MSG_CTL_1 + 6, r0  ; DLC (data length)=0 bytes,
                          ; DIR=0 (receive), XTD=1 (extended)
                         
   mov r0, #0000h         ; set arbitration registers for message
   mov MSG_CTL_2 + 2, r0  ; object 2.
   mov r0, #0080h         ; message object 2 accepts messages with 
   mov MSG_CTL_2 + 4, r0  ; the identifier 1.

                          ; set message configuration register 
   mov r0, #8Ch           ; for message object 1.
   mov MSG_CTL_2 + 6, r0  ; DLC (data length)=8 bytes,
                          ; DIR=1 (transmit), XTD=1 (extended)

                          ; initialization of the CAN controller 
   mov r0, #0000h         ; is completed. Clearing bit 6 (CCE) 
   mov CTL_ST, r0         ; inhibits further access to the 
                          ; bit timing register, clearing bit 0
   ret                    ; (INIT) completes the initialization.



send_rmt_rqst:            ; send remote request

   mov r0, #0E7FFh        ; MSG_CTL_1 <- 1110 0111 1111 1111 
   mov MSG_CTL_1, r0      ; TXRQ=1, CPUUPD=0

   calla cc_UC, set_rmt_status
   ret




receive_msg:

   mov r0, MSG_CTL_1      ; check message control register of 
   and r0, #0300h         ; object 1. See if NEWDAT=1  
   cmp r0, #0200h
   jmpr cc_NE, no_msg
   calla cc_UC, process_msg
   calla cc_UC, clr_rmt_status
no_msg:
   ret



process_msg:

   mov r0, #7             ; signal the reception of message
   calla cc_UC, blink     ; by blinking LED in P2.7
   
   mov r1, #MSG_CTL_1
   add r1, #7
   mov r2, #8
get_byte:
   movb rl0, [r1+]
   calla cc_UC, prthex
   cmpd1 r2, #1
   jmpr cc_SGT, get_byte

   mov r0, #0FDFFh        ; clear NEWDAT
   mov MSG_CTL_1, r0
   ret   




receiv_rmt_rqst:

   mov r0, MSG_CTL_2      ; check the message control register of
   and r0, #0F000h        ; object 2 to see if TXRQ=1 and RMTPND=1 
   cmp r0, #0A000h        ; (remote request received). 
   jmpr cc_NE, no_rmt_rqst   
   movb rl0, #1           ; if so, set remote_request_flag
   ret   
no_rmt_rqst:
   movb rl0, #0            ; if not, clear remote_request_flag
   ret




send_msg:

   mov r0, #0FEFFh        ; set NEWDAT
   mov MSG_CTL_2, r0

   mov r0, #55h           ; prepare the data to be transmitted
   movb MSG_CTL_2+7h, rl0
   movb MSG_CTL_2+8h, rl0
   movb MSG_CTL_2+9h, rl0
   movb MSG_CTL_2+0Ah, rl0
   movb MSG_CTL_2+0Bh, rl0
   movb MSG_CTL_2+0Ch, rl0
   movb MSG_CTL_2+0Dh, rl0
   movb MSG_CTL_2+0Eh, rl0

   mov r0, #0F7FFh        ; transmit
   mov MSG_CTL_2, r0      ; by clearing CPUUPD

   mov r0, #0FBFFh        ; set CPUUPD to prevent further 
   mov MSG_CTL_2, r0      ; transmissions without authorization

   ret




get_rmt_status:

   movb rl0, RMT_STATUS
   ret



set_rmt_status:

   movb rl1, #1
   movb RMT_STATUS, rl1
   ret



clr_rmt_status:

   movb rl1, #0
   movb RMT_STATUS, rl1
   ret



#include "r66util.inc"
